use std::sync::Arc;

use axum::{Extension, Json};
use foxchat::{
    http::ApiError,
    model::{http::guild::CreateGuildParams, user::PartialUser, Guild, channel::PartialChannel},
    FoxError,
};

use crate::{
    app_state::AppState,
    db::{channel::create_channel, guild::{create_guild, join_guild}},
    fed::FoxRequestData,
    model::user::User,
};

pub async fn post_guilds(
    Extension(state): Extension<Arc<AppState>>,
    request: FoxRequestData,
    Json(params): Json<CreateGuildParams>,
) -> Result<Json<Guild>, ApiError> {
    let user_id = request.user_id.ok_or(FoxError::MissingUser)?;
    let user = User::get(&state, &request.instance, &user_id).await?;

    let mut tx = state.pool.begin().await?;
    let guild = create_guild(&mut *tx, &user.id, &params.name).await?;
    let channel = create_channel(&mut *tx, &guild.id, "general", None).await?;

    join_guild(&mut *tx, &guild.id, &user.id).await?;

    tx.commit().await?;

    Ok(Json(Guild {
        id: guild.id,
        name: guild.name,
        owner: PartialUser {
            id: user.id,
            username: user.username,
            instance: user.instance.domain,
        },
        default_channel: PartialChannel {
            id: channel.id,
            name: channel.name,
        }
    }))
}
