pub mod error;
pub mod fed;
pub mod http;
pub mod model;
pub mod s2s;

pub use error::FoxError;
pub use fed::signature;

use chrono::{DateTime, Utc};
use ulid::Ulid;

/// Extracts a DateTime from a ULID.
/// This function should only be used on valid ULIDs (such as those used as primary keys), else it will fail and panic!
pub fn ulid_timestamp(id: &str) -> DateTime<Utc> {
    let ts = Ulid::from_string(id).expect("invalid ULID").timestamp_ms();
    let (secs, rem) = (ts / 1000, ts % 1000);
    let nsecs = rem * 1000000;

    DateTime::from_timestamp(
        secs.try_into().expect("converting secs to i64"),
        nsecs.try_into().expect("converting nsecs to i32"),
    )
    .expect("converting timestamp to DateTime")
}
